/**
 * Parking sensor
 * 
 * This project measures distances in 6 sides of the car.
 * On the display, we can see the car along 6 empty circles, that
 * means everything is fine.
 * If there is any object too close, a buzz will sound and a circle
 * from the approaching side will become full.
 * 
 * @author Stan Andrei
 * @version 1.0
 * @since 2022-05-24
 */

#include "U8glib.h"
#define buzzer 12
#define A0 14
#define A1 15
#define A2 16
#define A3 17
#define A4 18
#define A5 19
#define MIN_DISTANCE 10
#define RADIUS 5
#define NR_SENSORS 6
#define BUZZ_TIME 500
#define NATURAL unsigned long

U8GLIB_ST7920_128X64 u8g(13, 11, 10, U8G_PIN_NONE);
int x_circles[8], y_circles[8];
int trigPins[8];
int echoPins[8];
int distances[8];
NATURAL last_buzz = 0;
NATURAL ms;

int distance(int nr) {
  /* 
   * @param nr: number of sensor
   * @return int: distance to closest object in front of given sensor
   */
   
  // The sensor is triggered by a HIGH pulse of 10 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  digitalWrite(trigPins[nr], LOW);
  delayMicroseconds(5);
  digitalWrite(trigPins[nr], HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPins[nr], LOW);
 
  // Read the signal from the sensor: a HIGH pulse whose
  // duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(echoPins[nr], INPUT);
  long duration = pulseIn(echoPins[nr], HIGH);

  // Convert the time into a distance using formula in datasheet.
  return (duration / 2) / 29.1;
}

void draw() {
  /*
   * Draws on display the car shape and the 6 empty/full circles
   * showing the distance to a very close object.
   */
   
  // Prepare display
  u8g.setFont(u8g_font_6x10);
  u8g.setFontRefHeightExtendedText();
  u8g.setDefaultForegroundColor();
  u8g.setFontPosTop();

  // Draw shapes
  u8g.drawRBox(52, 12, 25, 40, 10); // Car

  for (int i = 0; i < NR_SENSORS; i++) {
    if (distances[i] < MIN_DISTANCE) {
      // If the distance is too short, draw a disc and activate buzzer
      u8g.drawDisc  (x_circles[i], y_circles[i], RADIUS);

      // Trigger buzzer for half a second
      digitalWrite(buzzer, HIGH);
      last_buzz = millis();
    }
    else
      u8g.drawCircle(x_circles[i], y_circles[i], RADIUS);
  }

  if (millis() - last_buzz > BUZZ_TIME)
    digitalWrite(buzzer, LOW);
}

void set_display() {
  // Define pins for display
#if defined(ARDUINO)
  pinMode(13, OUTPUT);           
  digitalWrite(13, HIGH);  
#endif
}

void set_coordinates() {
  // Coordinates for circles
  x_circles[0] = 83; x_circles[1] = 79; x_circles[2] = 64;
  x_circles[3] = 49; x_circles[4] = 45; x_circles[5] = 64;

  y_circles[0] = 32; y_circles[1] = 52; y_circles[2] = 58;
  y_circles[3] = 52; y_circles[4] = 32; y_circles[5] = 5;
}

void set_pins() {
  // Define pins for sensors
  trigPins[0] = 3;
  echoPins[0] = 2;
  
  trigPins[1] = 5;
  echoPins[1] = 4;
  
  trigPins[2] = A5;
  echoPins[2] = A4;
  
  trigPins[3] = A3;
  echoPins[3] = A2;
  
  trigPins[4] = A1;
  echoPins[4] = A0;
  
  trigPins[5] = 7;
  echoPins[5] = 6;
}

void set_inputs_outputs() {
  // Define inputs and outputs
  for (int i = 0; i < NR_SENSORS; i++) {
    pinMode(trigPins[i], OUTPUT);
    pinMode(echoPins[i], INPUT);
  }

  pinMode(buzzer, OUTPUT);
}

void setup() {
  set_display();
  set_coordinates();
  set_pins();
  set_inputs_outputs();

  // Set buzzer to silent
  digitalWrite(buzzer, LOW);
}

void loop() {
  // Compute distances
  for (int i = 0; i < NR_SENSORS; i++)
    distances[i] = distance(i);

  // picture loop  
  u8g.firstPage();
  do {
    draw();
  } while( u8g.nextPage() );
  
  // rebuild the picture after some delay
  delay(150);
}
